<?php  if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
function displayEANTextFieldOnFilterForm( $post_type ) 
{
	// Check for post type 
	if( $post_type == 'product' ) {
		$post_ean = '';
		if(isset($_GET['post_ean'])){
			$post_ean = $_GET['post_ean'];
		}
		// Text field to display on filter list on product page in wp-admin
		echo '<input type="text" name="post_ean" placeholder="Filter by EAN" autocomplete="off" value="'.$post_ean.'">';
	}
}
add_action( 'restrict_manage_posts', 'displayEANTextFieldOnFilterForm' );

function apply_filter_on_EAN( $query ) 
{
	if ( $query->is_admin && isset( $_GET['post_ean'] ) && $_GET['post_ean'] != '' && $_GET['post_type'] == 'product' ) {
		// Query to filter data with EAN in wp-admin
		$meta_key_query = array(
			array(
				'key'     => '_mantella_ean_number',
				'value'   => esc_attr( $_GET['post_ean'] ),
			)
		);
		$query->set( 'meta_query', $meta_key_query );
	}
}
add_action( 'pre_get_posts', 'apply_filter_on_EAN' );

function add_admin_column($column_title, $post_type, $cb){

    // Column Header
    add_filter( 'manage_' . $post_type . '_posts_columns', function($columns) use ($column_title) {
        $columns[ sanitize_title($column_title) ] = $column_title;
        return $columns;
    } );

    // Column Content
    add_action( 'manage_' . $post_type . '_posts_custom_column' , function( $column, $post_id ) use ($column_title, $cb) {

        if(sanitize_title($column_title) === $column){
            $cb($post_id);
        }

    }, 10, 2 );
}
add_admin_column(__('EAN'), 'product', function($post_id){
    echo get_post_meta( $post_id , '_mantella_ean_number' , true ); 
});

//new field added for the variations
add_action( 'woocommerce_product_after_variable_attributes', 'mantella_ean_code_to_variations', 10, 3 ); 
function mantella_ean_code_to_variations( $loop, $variation_data, $variation ) {
	
	woocommerce_wp_text_input( array(
	'id' => 'mantella_ean_code['.$loop.']',
	'class' => 'short drpship-code',
	'label' => __( 'EAN Code:', 'dropship-tool-by-mantella' ),
	'value' => get_post_meta( $variation->ID, 'mantella_ean_code', true )
	));
	woocommerce_wp_text_input( array(
		'id' => 'mantella_ean_brand['.$loop.']',
		'class' => 'short drpship-brand',
		'label' => __( 'Brand:', 'dropship-tool-by-mantella' ),
		'value' => get_post_meta( $variation->ID, 'mantella_ean_brand', true )
	));
}
 
// Save custom field on product variation save 
add_action( 'woocommerce_save_product_variation', 'mantella_save_mantella_ean_code_variations', 10, 2 ); 
function mantella_save_mantella_ean_code_variations( $variation_id, $i ) {
   $mantella_ean_code = $_POST['mantella_ean_code'][$i];
   $mantella_ean_brand = $_POST['mantella_ean_brand'][$i];
   if ( isset( $mantella_ean_code ) ) update_post_meta( $variation_id, 'mantella_ean_code', esc_attr( $mantella_ean_code ) );
   if ( isset( $mantella_ean_brand ) ) update_post_meta( $variation_id, 'mantella_ean_brand', esc_attr( $mantella_ean_brand ) );
}
// Store custom field value into variation data 
add_filter( 'woocommerce_available_variation', 'mantella_ean_code_variation_data' ); 
function mantella_ean_code_variation_data( $variations ) {
    global $wpdb;
    $record = $wpdb->get_results("SELECT `ean_show_in_front`, `brand_show_in_front` FROM " . $wpdb->prefix . "license_key");

        $variations['mantella_ean_code']   = '';
    if($record[0]->ean_show_in_front == 'Yes' && get_post_meta( $variations[ 'variation_id' ], 'mantella_ean_code', true )){
        $variations['mantella_ean_code']   = '<div class="woocommerce_mantella_ean_code"><span class="dropshipcustom'.$variations[ 'variation_id' ].'"> EAN Code: ' . get_post_meta( $variations[ 'variation_id' ], 'mantella_ean_code', true ) . '</span></div>';
    }
        $variations['mantella_ean_brand']  = '';
    if($record[0]->brand_show_in_front == 'Yes' && get_post_meta( $variations[ 'variation_id' ], 'mantella_ean_brand', true )){
        $variations['mantella_ean_brand']  = '<div class="woocommerce_mantella_ean_brand"><span class="dropshipcustom'.$variations[ 'variation_id' ].'"> Brand: ' . get_post_meta( $variations[ 'variation_id' ], 'mantella_ean_brand', true ) . '</span></div>';
    }
        $variations['mantella_ean_number'] = '';
    if(get_post_meta( $variations[ 'variation_id' ], 'mantella_ean_number', true )){
        $variations['mantella_ean_number'] = get_post_meta( $variations[ 'variation_id' ], 'mantella_ean_code', true );
    }
        
    return $variations;
}
add_action('admin_head','custom_js_for_variation');
function custom_js_for_variation() { ?>    
    <script>
    jQuery(document).ready(function(){

		jQuery( document ).ajaxComplete(function() {
			jQuery(".drpship-code").parents('p').addClass('form-row form-row-first');
			jQuery(".drpship-brand").parents('p').addClass('form-row form-row-last');			
		});
		jQuery(".dropship_tool").hide();
        jQuery( "#product-type" ).on( "change", function () {
            jQuery("#mantella_extra_product_data").hide();
            jQuery(".cust_tab_flex").hide();
						
            if(jQuery(this).val() == 'variable'){ 
                jQuery(".mantella-extra-tab_tab").hide();
            }else if(jQuery(this).val() == 'simple'){
                jQuery("#mantella_extra_product_data").show();
                jQuery(".cust_tab_flex").show();
				jQuery(".mantella-extra-tab_tab").show();
            }
        }); 

    });
    </script>
<?php
}

//Show in webfront
add_action('woocommerce_before_add_to_cart_button', 'mantella_EAN_display_brand_before_addtocart');
function mantella_EAN_display_brand_before_addtocart() {
	echo "<div class='dropship-brand-code' style='display:none'></div>";	
}
//on change set the value of variations
add_action( 'woocommerce_before_single_variation', 'action_wc_before_single_variation' );
function action_wc_before_single_variation() {
    ?>
    <script type="text/javascript">
    (function(jQuery){
        jQuery('form.variations_form').on('show_variation', function(event, data){
            jQuery('#ean_number').val(data.mantella_ean_number);
			jQuery('.dropship-brand-code').html(data.mantella_ean_code+data.mantella_ean_brand);			
			jQuery('.dropship-brand-code').show();
        });
    })(jQuery);
    
    jQuery('.reset_variations').click(function(){
        jQuery( ".dropship-brand-code" ).hide();
    });
    </script>
    <?php
}