<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class DropshipAddtoCart
{
    private $plugin_name;
    private $version;
    private $license_no;
    private $domain;
    private $api_domain;
    private $data;

    public function __construct($dataArray)
    {
        $this->plugin_name = $dataArray['plugin_name'];
        $this->version = $dataArray['plugin_version'];
        $this->domain = $dataArray['plugin_domain'];
        $this->api_domain = $dataArray['api_domain'];
        $this->data = $dataArray;

        add_action( 'woocommerce_before_add_to_cart_button', array($this,'mantella_add_text_field'));
        add_filter( 'woocommerce_add_to_cart_validation', array($this,'mantella_woocommerce_add_to_cart_validation'), 10, 3 );
        add_filter( 'woocommerce_add_cart_item_data', array($this,'mantella_woocommerce_add_cart_item_data'), 10, 3 );
        add_filter('woocommerce_get_cart_item_from_session', array($this,'get_user_custom_data_session'), 1, 3 );
        add_filter('woocommerce_checkout_cart_item_quantity', array($this,'wdm_add_user_custom_option_from_session_into_cart'),1,3);
        add_filter('woocommerce_cart_item_price', array($this,'wdm_add_user_custom_option_from_session_into_cart'),1,3);
		add_action('woocommerce_new_order_item',array($this,'wdm_add_values_to_order_item_meta'),1,2);
    }

    //Add EAN NO for checking stock
    public function mantella_add_text_field()
    {
        global $product;
        if($product->product_type=='variable') {
            $available_variations = $product->get_available_variations();
            $variation_id=$available_variations[0]['variation_id'];
            $ean_number = get_post_meta($variation_id, "mantella_ean_code", true);
        }else{
            $ean_number = get_post_meta($product->get_id(), "_mantella_ean_number", true);
        }

        
        $allowedProduct = $product->backorders_allowed();
        if($allowedProduct==true){$allowedProduct=1;}else{$allowedProduct=0;}
        ?>
        <input type="hidden" name='allowedProduct' id='allowedProduct' value='<?php echo $allowedProduct; ?>'>
        <input type="hidden" name='ean_number' id='ean_number' value='<?php echo $ean_number; ?>'>
    <?php
    }

    //Get stock with EAN No from web portal
    public function checkStockByEAN($ean_number, $domain, $license_no)
    {
       $url = $this->api_domain . "feeds/". $domain ."/". $license_no ."/stock/" .$ean_number;
    
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
        $result = curl_exec($ch);
        $json = json_decode($result, true);

        if( isset( $json['stock'] ) )
            return $json['stock'];
        else
            return 'error';
    }

    public function CheckLicenseKeys($license_no, $domain)
    {
        $url = $this->api_domain . "feeds/" . $domain . "/" . $license_no . "/feed/total";

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $url);
        $l_result = curl_exec($ch);
        curl_close($ch);
        $license_key_status = json_decode($l_result, true);
        return $license_key_status;
    }

    //Check stock with EAN No from web portal
    public function mantella_woocommerce_add_to_cart_validation( $true, $product_id, $quantity )
    {
        global $wpdb;
        $this->record       = $wpdb->get_results("select license_no from " . $wpdb->prefix . "license_key");
        $this->license_no   = $this->record[0]->license_no;
        $ean_number         = get_post_meta($product_id, "_mantella_ean_number", true);
        $allowedProduct     = (isset($_POST['allowedProduct'])) ? $_POST['allowedProduct'] : "0";

        if(!empty($ean_number)){
            $stock = $this->checkStockByEAN($ean_number, $this->domain, $this->license_no);

            if( $stock == 'error' && !is_numeric($stock) )
            {
                return true;
            }
            else
            {
                $product = wc_get_product($product_id);  
                if($product && $product->get_type() === 'variable')
                {
                    $product_id = $wpdb->get_var("SELECT post_id FROM $wpdb->postmeta WHERE meta_key = 'mantella_ean_code' AND meta_value = '" . $ean_number . "'");
                }

                if ($allowedProduct == "1")
                {
                    update_post_meta($product_id, '_stock', $stock);
                    update_post_meta($product_id, '_stock_status', 'instock');

                    //update attribute table if needed
                    $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                    $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 1 ), array( 'product_id' =>  $product_id) );

                    return $true;
                }
                else
                {
                    if ($stock >= $quantity)
                    {
                        update_post_meta($product_id, '_stock', $stock);
                        update_post_meta($product_id, '_stock_status', 'instock');

                        //update attribute table if needed
                        $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                        $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 1 ), array( 'product_id' =>  $product_id) );

                        return $true;
                    }
                    else
                    {
                        update_post_meta($product_id, '_stock', $stock);
                        update_post_meta($product_id, '_stock_status', 'outofstock');

                        //update attribute table if needed
                        $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                        $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 0 ), array( 'product_id' =>  $product_id) );

                        return $true;
                    }

                }

            }
        }
        
        if( $_POST['ean_number'] )
        {
            $cart_item_meta['ean_number'] = $_POST['ean_number'];
            $ean_number = $_POST['ean_number'];
            $allowedProduct = $_POST['allowedProduct'];
            $license_key_status = $this->CheckLicenseKeys($this->license_no, $this->domain);

            if(isset($license_key_status['access']) && $license_key_status['access']=='denied')
            {
                return $true;
            }
            else
            {
                $stock = $this->checkStockByEAN($ean_number, $this->domain, $this->license_no);

                if( $stock == 'error' && !is_numeric($stock) )
                {
                    return true;
                }
                else
                {
                    $product = wc_get_product($product_id);  
                    if($product && $product->get_type() === 'variable')
                    {
                        $product_id = $wpdb->get_var("SELECT post_id FROM $wpdb->postmeta WHERE meta_key = 'mantella_ean_code' AND meta_value = '" . $ean_number . "'");
                    }
                    
                    if ($allowedProduct == "1")
                    {
                        update_post_meta($product_id, '_stock', $stock);
                        update_post_meta($product_id, '_stock_status', 'instock');
                        return $true;
                    }
                    else
                    {
                        if ($stock >= $quantity)
                        {
                            update_post_meta($product_id, '_stock', $stock);
                            update_post_meta($product_id, '_stock_status', 'instock');
                            return $true;
                        }
                        else
                        {
                            update_post_meta($product_id, '_stock', $stock);
                            update_post_meta($product_id, '_stock_status', 'outofstock');
                            return $true;
                        }

                    }

                }

            }

        }
        else
        {
            return $true;
        }
    }

    //Add items meta to woocommerce cart_item for further use
    public function mantella_woocommerce_add_cart_item_data( $cart_item_data, $product_id, $variation_id )
    {
        global $woocommerce;
        session_start();
        $cart_item_meta['ean_number'] = $_POST['ean_number'];
        $_SESSION['ean_number'] = $_POST['ean_number'];

        if (isset($_SESSION['ean_number']))
        {
            $option = $_SESSION['ean_number'];
            $new_value = array('ean_number' => $option);
        }

        if(empty($option))
            return $cart_item_data;
        else
        {
            if(empty($cart_item_data))
                return $new_value;
            else
                return array_merge($cart_item_data,$new_value);
        }

        unset($_SESSION['ean_number']);
        //Unset our custom session variable, as it is no longer needed.

    }

    //Set data to woocommerce cart object for further use
    public function get_user_custom_data_session( $item, $values, $key )
    {
        if (array_key_exists( 'ean_number', $values ) )
        {
            $item['ean_number'] = $values['ean_number'];
        }
        return $item;
    }

    //Display to cart checkout page
    public function wdm_add_user_custom_option_from_session_into_cart($product_name, $values, $cart_item_key )
    {
        if($values['ean_number'])
        {
            $return_string = $product_name . "</a><dl class='variation'>";
            $return_string .= "<table class='wdm_options_table' id='" . $values['product_id'] . "'>";
            $return_string .= "</table></dl>";
            return $return_string;
        }
        else
        {
            return $product_name;
        }
    }

    // Adding Order item
    public function wdm_add_values_to_order_item_meta($item_id, $values)
    {
        global $woocommerce,$wpdb;
        $user_custom_values = $values['wdm_user_custom_data_value'];
        if(!empty($user_custom_values))
        {
            wc_add_order_item_meta($item_id,'wdm_user_custom_data',$user_custom_values);
        }
    }

}