<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
set_time_limit(-1); //0 or -1 for unlimited

class DropshipImportUpdateMethod
{
    private $plugin_name;
    private $version;
    private $domain;
    private $api_domain;
    private $data;

    public function __construct($dataArray)
    {
        $this->plugin_name      = $dataArray['plugin_name'];
        $this->version          = $dataArray['plugin_version'];
        $this->domain           = $dataArray['plugin_domain'];
        $this->api_domain       = $dataArray['api_domain'];
        $this->data             = $dataArray;
    }

    function update_import_method_from_cron_new($lang, $record, $data, $post_id, $b)
    {
        global $wpdb;

        include_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        include_once plugin_dir_path(dirname(__FILE__)) . 'core/wp-product-import.php';

        $importObj = new DropshipProductImport($this->data);
    
        if (!empty($data)) {

            $product                = wc_get_product($post_id);
            $array_updlist          = NULL;
            $this->sale_price       = $product->sale_price;
            $this->import_settings  = unserialize($record[0]->import_settings);
            //condition for the variable product

            if($product && $product->get_type() === 'variation')
            {
                /**
                 * Update stock
                 */
                if ($this->import_settings['stock_status_update'] == 'Yes') 
                {
                    //get parent id
                    $product_parent_id      = $product->parent_id;
                    $parentproduct          = wc_get_product($product_parent_id);
                    $childeren_products     = $parentproduct->get_children();
                    //total addition of variations stock
                    $total_stock_for_parent = 0;
                    if($childeren_products && count($childeren_products) > 0){
                        foreach($childeren_products as $vproduct_id){
                            $total_stock_for_parent += get_post_meta($vproduct_id, "_stock", true);
                        }
                    }

                    update_post_meta($product_parent_id, '_manage_stock', "Yes");

                    if($total_stock_for_parent > 0){
                        update_post_meta($product_parent_id, '_stock_status', 'instock');
                        update_post_meta($product_parent_id, '_stock', $total_stock_for_parent);
                    }else{
                        update_post_meta($product_parent_id, '_stock_status', 'outofstock');
                        update_post_meta($product_parent_id, '_stock', $total_stock_for_parent);
                    }

                    update_post_meta($post_id, '_manage_stock', "Yes");
                    update_post_meta($post_id, '_stock', $data['stock']);

                    if ($data['stock'] < 1){
                        update_post_meta($post_id, '_stock_status', 'outofstock');

                        $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                        $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 0 ), array( 'product_id' =>  $post_id) );
                    }
                    else{
                        update_post_meta($post_id, '_stock_status', 'instock');

                        $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                        $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 1 ), array( 'product_id' =>  $post_id) );
                    }
                }
                
            }
            else
            {
                /**
                 * Attribute Product Details
                 */
                if ($this->import_settings['product_attributes_status_update'] == 'Yes') 
                {

                    /** preparations for attribute creation
                    Check if the custom attributes has been set
                     */

                    if( $this->import_settings['attr_choose'] == 'No')
                        $array_attr = NULL;
                    else {
                        $expl       = explode('|', $this->import_settings['custom_attributes']);
                        $array_attr = array();
                        foreach( $expl as $ex ) { $array_attr[] = $ex; }
                    }

                    /** check if brand also needs to be added */
                    if ($this->import_settings['store_brand_as_attribute_update'] == 'Yes' && isset($data['brand']) && $data['brand'] != '') {
                        $brand_attr['id']       = $this->post_id;
                        $brand_attr['name_en']  = 'Brand';
                        $brand_attr['name_nl']  = 'Merk';
                        $brand_attr['name_de']  = 'Marke';
                        $brand_attr['value_en'] = $data['brand'];
                        $brand_attr['value_nl'] = $data['brand'];
                        $brand_attr['value_de'] = $data['brand'];

                        if ($data['attributes']) {
                            array_push($data['attributes'], $brand_attr);
                        } else {
                            $data['attributes'] = [];
                            array_push($data['attributes'], $brand_attr);
                        }
                    }

                    if ($this->import_settings['attribute_type'] == 'global')
                        $b = $importObj->importProductAttributes($data, $lang, $post_id, $b, $array_attr);
                    else
                    {
                        if (isset($data['attributes']))
                        {
                            $product_attributes = NULL;
                            $a = 0;

                            foreach ($data['attributes'] as $attr) {

                                if( $array_attr != NULL ) {
                                    if( !in_array(strtolower($attr['name_' . $lang]), $array_attr) ) {
                                        continue;
                                    }
                                }

                                $product_attributes[$a] = array(
                                    'name'          => $attr['name_'.$lang],
                                    'value'         => $attr['value_'.$lang],
                                    'position'      => 1,
                                    'is_visible'    => 1,
                                    'is_variation'  => 1,
                                    'is_taxonomy'   => 0
                                );
                                $a++;
                            }
                            update_post_meta($post_id, '_product_attributes', $product_attributes);
                        }
                    }
                    
                }
           
                /**
                 * Update Shipping Classes
                 */
                if ($this->import_settings['shipping_class_status_update'] == 'Yes') 
                {
                    if($data['shipping_size']=="XS")
                        wp_set_post_terms($post_id, array("tom_bv_xs"), 'product_shipping_class');
                    elseif($data['shipping_size']=="S")
                        wp_set_post_terms($post_id, array("tom_bv_s"), 'product_shipping_class');
                    elseif($data['shipping_size']=="M")
                        wp_set_post_terms($post_id, array("tom_bv_m"), 'product_shipping_class');
                    elseif($data['shipping_size']=="L")
                        wp_set_post_terms($post_id, array("tom_bv_l"), 'product_shipping_class');
                    elseif($data['shipping_size']=="XL")
                        wp_set_post_terms($post_id, array("tom_bv_xl"), 'product_shipping_class');
                }

                /**
                 * Update name, description, categories and attributes per language
                 */
                //condition to append brand with product name
                if ($this->import_settings['title_status_update'] == 'Yes')
                {
                    if ($record[0]->brand_with_title_status == 'Yes')
                    {
                        $this->product_name = ucfirst($data['brand'].' '.strtolower($data['name_'.$lang]));                         
                    }
                    else
                    {
                        $this->product_name = ucfirst(strtolower($data['name_'.$lang]));  
                    }
                    
                    $array_updlist['post_title'] = $this->product_name;
                }

                if ($this->import_settings['description_status_update'] == 'Yes') 
                {
                    $this->product_desc = $data['description_'.$lang];
                    $array_updlist['post_content'] = $this->product_desc;
                }

                if ($this->import_settings['shortdescription_status_update'] == 'Yes')
                {

                    if( $lang == 'nl' )
                        $desc_arr = explode("Specificaties", $data['description_'.$lang], 2);
                    elseif( $lang == 'en' )
                        $desc_arr = explode("Specifications", $data['description_' . $lang], 2);
                    elseif( $lang == 'de' )
                        $desc_arr = explode("Spezifikationen", $data['description_'.$lang], 2);

                    $prod_excerpt = $desc_arr[0];
                    $prod_excerpt = str_replace( '<strong>', '', $prod_excerpt);
                    $prod_excerpt = str_replace( '</strong>', '', $prod_excerpt);

                    $array_updlist['post_excerpt'] = $prod_excerpt;
                }

                if ($this->import_settings['url_status_update'] == 'Yes') 
                {
                    $slug_url = sanitize_title($this->product_name);
                    $array_updlist['post_name'] = $slug_url;
                }

                /**
                 * Categories
                 */
                if($array_updlist)
                {
                    $table_name = $wpdb->prefix . 'posts';
                    $wpdb->update( $table_name, $array_updlist, array('ID' => $post_id));
                }

                if ($this->import_settings['categories_status_update'] == 'Yes') 
                {
                    wp_set_object_terms($post_id, $data['categories']['parent_name_'.$lang], 'product_cat');
                    wp_insert_term($data['categories']['child_name_'.$lang], 'product_cat', array('parent' => term_exists($data['categories']['parent_name_'.$lang], 'product_cat')['term_id']));
                    wp_set_object_terms($post_id, [$data['categories']['parent_name_'.$lang], $data['categories']['child_name_'.$lang]], 'product_cat', true);
                    wp_set_object_terms($post_id, 'simple', 'product_type');
                }

                /**
                 * Update pricing
                 */
                if ($this->import_settings['prices_status_update'] == 'Yes') 
                {
                    if ($this->sale_price) {
                        update_post_meta($post_id, '_sale_price', $this->sale_price);
                        update_post_meta($post_id, '_price', $this->sale_price);
                        update_post_meta($post_id, '_regular_price', $data['consumer_price']);
                    } else {
                        update_post_meta($post_id, '_sale_price', '');
                        update_post_meta($post_id, '_price', $data['consumer_price']);
                        update_post_meta($post_id, '_regular_price', $data['consumer_price']);
                    }
                }

                /**
                 * Update other information
                 */
                if ($this->import_settings['stock_status_update'] == 'Yes') 
                {
                    update_post_meta($post_id, '_manage_stock', "Yes");
                    update_post_meta($post_id, '_stock', $data['stock']);

                    if ($data['stock'] < 1) {
                        update_post_meta($post_id, '_stock_status', 'outofstock');

                        $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                        $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 0 ), array( 'product_id' =>  $post_id) );
                    }
                    else{
                        update_post_meta($post_id, '_stock_status', 'instock');

                        $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                        $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 1 ), array( 'product_id' =>  $post_id) );
                    }
                }
              
                update_post_meta($post_id, '_mantella_ean_number_show_in_front', $record[0]->ean_show_in_front);
                update_post_meta($post_id, '_mantella_brand_show_in_front', $record[0]->brand_show_in_front);
                
                if ($this->import_settings['brand_status_update'] == 'Yes') 
                {
                    update_post_meta($post_id, '_mantella_brand', $data['brand']);
                }

                if ($this->import_settings['image_status_update'] == 'Yes') 
                {
                    //Image uploading
                    if( isset( $data['media'] ) ) // upload multiple images
                    {
                        $gallery_img    = NULL;  $c = 1;
                        foreach( $data['media'] as $media )
                        {
                            if( $media['type'] == 'image')
                            {
                                //check image exists
                                $existsresponse = $importObj->checkFileExists($post_id, $media['url']);
                                if($existsresponse == 0){

                                    $attach_id_img = $importObj->uploadProductImagesCronNow($post_id, $media['url']);
                                    if( $c > 1)
                                    {
                                        if($c == 2)
                                            $gallery_img = $attach_id_img;
                                        else
                                            $gallery_img = $gallery_img .','. $attach_id_img;
                                    } else
                                        set_post_thumbnail($post_id, $attach_id_img);
                                }
                            }
                            $c++;
                        }
                        update_post_meta($post_id, '_product_image_gallery', $gallery_img);
                    }
                    else
                    {
                        $gallery_img    = NULL;
                        $existsresponse = $importObj->checkFileExists($post_id, $data['image_1']);
                        if($existsresponse == 0){
                            $attach_id_img1 = $importObj->uploadProductImagesCronNow($post_id, $data['image_1']);
                            set_post_thumbnail($post_id, $attach_id_img1);
                        }
                        
                        if ($data['image_2'] != NULL)
                        {
                            $existsresponse = $importObj->checkFileExists($post_id, $data['image_2']);
                            if($existsresponse == 0){
                                $attach_id_img2 = $importObj->uploadProductImagesCronNow($post_id, $data['image_2']);
                                $gallery_img = $attach_id_img2;
                                update_post_meta($post_id, '_product_image_gallery', $gallery_img);
                            }
                        }
                    }            
                
                    unset($gallery_img);
                    unset($attach_id_img1);
                    if($attach_id_img2)
                        unset($attach_id_img2);
                }

                wc_delete_product_transients($post_id);

                if ($this->import_settings['attribute_type'] == 'global')
                    return $b;
            }
        }
    }
    
}