<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly
error_reporting(0); //added hide error reporting in case of updating pluging

class DropshipUpdate
{
    private $plugin_name;
    private $version;
    private $domain;
    private $api_domain;
    private $data;
    private $plugin_slug;
    private $foldername;
    private $dirsname;

    public function __construct($dataArray)
    {
        $this->plugin_name  = $dataArray['plugin_name'];
        $this->version      = $dataArray['plugin_version'];
        $this->domain       = $dataArray['plugin_domain'];
        $this->api_domain   = $dataArray['api_domain'];
        $this->data         = $dataArray;
        $this->plugin_slug  = 'dropship-tool-by-mantella';

        if($_SERVER['HTTP_HOST']=='localhost') {
            $this->dirsname     = explode('\\',plugin_dir_path(__FILE__));
            $this->foldername   = @end($this->dirsname);
        }
        else {
            $this->dirsname     = explode('/', PLUGIN_DIR_URL);
            $this->foldername   = @end(array_filter($this->dirsname)).'/';
        }

        add_filter('plugins_api', array($this,'dropship_plugin_info'), 20, 3);
        add_filter('site_transient_update_plugins', array($this,'dropship_push_update'));
        add_action('upgrader_process_complete', array($this,'dropship_after_update'), 10, 2 );
    }

    public function dropship_plugin_info( $res, $action, $args )
    {
        // do nothing if this is not about getting plugin information
        if( 'plugin_information' !== $action ) {
            return false;
        }

        $this->plugin_slug = 'dropship-tool-by-mantella'; // we are going to use it in many places in this function

        // do nothing if it is not our plugin
        if( $this->plugin_slug !== $args->slug )
        {
            return false;
        }

        // trying to get from cache first
        if( false == $remote = get_transient( 'dropship_update_' . $this->plugin_slug ) )
        {
            // info.json is the file with the actual plugin information on your server
            $remote = wp_remote_get( 'https://plugins.dropshiptool.eu/mantella/plugin_updater/mantella_info.json', array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    ) )
            );

            if ( ! is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && ! empty( $remote['body'] ) )
            {
                set_transient( 'dropship_update_' . $this->plugin_slug, $remote, 43200 ); // 12 hours cache
            }

        }

        if( ! is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && ! empty( $remote['body'] ) )
        {
            $remote                 = json_decode( $remote['body'] );
            $res                    = new stdClass();
            $res->name              = $remote->name;
            $res->slug              = $this->plugin_slug;
            $res->version           = $remote->version;
            $res->tested            = $remote->tested;
            $res->requires          = $remote->requires;
            $res->author            = $remote->author;
            $res->author_profile    = $remote->author_homepage;
            $res->download_link     = $remote->download_url;
            $res->trunk             = $remote->download_url;
            $res->requires_php      = $remote->requires;
            $res->last_updated      = $remote->last_updated;
            $res->sections = array(
                'description'   => $remote->sections->description,
                'installation'  => $remote->sections->installation,
                'changelog'     => $remote->sections->changelog
            );

            if( !empty( $remote->sections->screenshots ) )
            {
                $res->sections['screenshots'] = $remote->sections->screenshots;
            }

            $res->banners = array(
                'low' => 'https://plugins.dropshiptool.eu/mantella/plugin_updater/image/dropshiptool-by-mantella-logo.jpg',
                'high' => 'https://plugins.dropshiptool.eu/mantella/plugin_updater/image/dropshiptool-by-mantella-logo.jpg'
            );
            return $res;
        }
        return false;
    }

    public function dropship_push_update( $transient )
    {
        global $wpdb;
        include_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        $table_name = $wpdb->prefix . 'license_key';

        //Add column if not present.
        if($this->checkColumnExists( 'perpage', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD perpage INT(10) NOT NULL ");
        }
        if($this->checkColumnExists( 'execution_time', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD execution_time INT(10) NOT NULL ");
        }
        if($this->checkColumnExists( 'time_var', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD time_var INT(10) NOT NULL ");
        }
        if($this->checkColumnExists( 'ean_show_in_front', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD ean_show_in_front varchar(5) DEFAULT NULL ");
        }
        if($this->checkColumnExists( 'brand_show_in_front', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD brand_show_in_front varchar(5) DEFAULT NULL ");
        }
        if($this->checkColumnExists( 'product_attributes_status', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD product_attributes_status varchar(5) DEFAULT NULL ");
        }
        if($this->checkColumnExists( 'brand_with_title_status', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD brand_with_title_status varchar(5) DEFAULT NULL ");
        }
        if($this->checkColumnExists( 'title_status', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD title_status varchar(5) DEFAULT NULL ");
        }
        if($this->checkColumnExists( 'import_settings', $table_name ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_name ." ADD import_settings TEXT DEFAULT NULL ");
        }


        //Update dropship_track_logs from version 3.0.0
        $table_logs = $wpdb->prefix . 'dropship_track_logs';

        if($this->checkColumnExists( 'log_type', $table_logs ) == 0 ) {
            $wpdb->query("ALTER TABLE ". $table_logs ." ADD log_type varchar(100) DEFAULT NULL ");
        }

        //check index exists in table or not
        $indexresult = $wpdb->get_results("SELECT COUNT(1) indexCount FROM INFORMATION_SCHEMA.STATISTICS
        WHERE table_schema=DATABASE() AND table_name='{$wpdb->prefix}postmeta' AND INDEX_NAME='meta_value'");

        //Add index in postmeta table if not exists
        if($indexresult[0]->indexCount == 0){
            $sql = "CREATE INDEX meta_value ON {$wpdb->prefix}postmeta (meta_value(128))";
            $wpdb->query ($sql) ;
        }

        if ( empty($transient->checked ) )
        {
            return $transient;
        }

        if( false == $remote = get_transient( 'dropship_upgrade_'. $this->plugin_slug ) )
        {
            $remote = wp_remote_get( 'https://plugins.dropshiptool.eu/mantella/plugin_updater/mantella_info.json', array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    ) )
            );

            if ( !is_wp_error( $remote ) && isset( $remote['response']['code'] ) && $remote['response']['code'] == 200 && !empty( $remote['body'] ) )
            {
                set_transient( 'dropship_upgrade_'.$this->plugin_slug, $remote, 200 ); // 12 hours cache
            }
        }

        if( $remote ) {
            if( isset( $remote->errors ) && $remote->errors != '' )
            {
                foreach ( $remote->errors as $err )
                {

                    if( $err[0] == 'cURL error 60: SSL certificate problem: certificate has expired')
                        $err_msg_2 = '. <strong>Ask your host to update the cURL software SSL Verion. It needs to be 1.1.x or higher.</strong>';
                    else
                        $err_msg_2 = '';
                    ?>
                    <div class="notice notice-error is-dismissible">
                        <p><strong>Dropship Plugin error:</strong> <?php echo $err[0] . $err_msg_2;
                            ?></p>
                    </div>
                    <?php
                    break;
                }

            } elseif(isset($remote['body']) && !is_wp_error($remote['body'])){
                $remote = json_decode($remote['body']);
                if(is_object($remote)){
                    //if( $remote && version_compare( $pv, $remote->version, '<' ) && version_compare($remote->requires, get_bloginfo('version'), '<' ) ) {
                    if( $remote && version_compare( DROPSHIP_TOOL_BY_MANTELLA_VERSION, $remote->version, '<' ) ) {
                        $res = new stdClass();
                        $res->slug =  $this->plugin_slug;
                        $res->plugin = $this->foldername.'mantella.php'; // it could be just YOUR_PLUGIN_SLUG.php if your plugin doesn't have its own directory
                        $res->new_version = $remote->version;
                        $res->tested = $remote->tested;
                        $res->package = $remote->download_url;
                        $transient->response[$res->plugin] = $res;
                    }
                }
            }
        }
        return $transient;
    }

    public function dropship_after_update( $upgrader_object, $options )
    {
        //send data to update shop info
        include_once plugin_dir_path(dirname(__FILE__)) . 'core/mantella_sendtoportal.php';

        $dsp = new DropshipSendtoPortal($this->data);
        $dsp->sendtoAPI();

        if ( $options['action'] == 'update' && $options['type'] === 'plugin' )
        {
            delete_transient( 'dropship_upgrade_'.$this->plugin_slug );
        }
    }

    public function checkColumnExists( $columnname, $table_name = NULL )
    {
        include_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        global $wpdb;

        if($table_name == NULL)
            return 1; //return value that doesnt create column again

        $check_column = (array) $wpdb->get_results("SELECT count(COLUMN_NAME) FROM INFORMATION_SCHEMA.COLUMNS 
                        WHERE TABLE_SCHEMA=DATABASE() AND TABLE_NAME = '$table_name' 
                        AND COLUMN_NAME = '$columnname'"  )[0];

        $check_column = (int) array_shift($check_column);

        return $check_column;
    }

}
