<?php
if (!defined('ABSPATH')) exit; // Exit if accessed directly

ini_set('max_execution_time', -1); // 0 or -1 for Unlimited
set_time_limit(-1); //0 or -1 for unlimited

class DropshipProductImport
{
    private $plugin_name;
    private $version;
    private $domain;
    private $api_domain;
    private $data;
    private $product_name;
    private $product_desc;
    private $post_id;
    private $post;
    private $mantella_ean_number_show_in_front;
    private $mantella_brand_show_in_front;

    public function __construct($dataArray)
    {
        $this->plugin_name  = $dataArray['plugin_name'];
        $this->version      = $dataArray['plugin_version'];
        $this->domain       = $dataArray['plugin_domain'];
        $this->api_domain   = $dataArray['api_domain'];
        $this->data         = $dataArray;

        add_action('init', array($this, 'callCronSchedulesAfterPluginActivate'));
    }

    /**
     * Setup schedular for import/update products
     */

    public function mantella_custom_productimport_cronjob_schedule_now($schedules)
    {
        //fetch pages record from db
        global $wpdb;
        $record = $wpdb->get_results("SELECT `perpage`, `import_settings` FROM " . $wpdb->prefix . "license_key");
        if (isset($record[0]->import_settings)) {
            $this->import_settings = unserialize($record[0]->import_settings);
        }

        if ($record[0]->perpage != 0 && $this->import_settings['cron_repeat_time'] == 20)
            //$croninterval = $record[0]->perpage*2;
            $croninterval = $this->import_settings['cron_repeat_time'];
        else
            $croninterval = 60;

        $schedules['every_three_min_cron_now'] = array(
            'interval' => $croninterval,
            'display' => __('Every ' . $croninterval . ' seconds'),
        );
        return $schedules;
    }

    public function mantella_cron_productimport_function_now()
    {
        global $wpdb;
        include_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        $record     = $wpdb->get_results("SELECT `id`,`license_no`,`total_feed`,`language` FROM " . $wpdb->prefix . "license_key");
        $language   = $record[0]->language;
        $license_no = $record[0]->license_no;
        $domain     = $this->data["plugin_domain"];
        $rec_count  = $wpdb->get_results("select count(*) as tot from " . $wpdb->prefix . "dropship_import_cron_dtl_temp");
        $count      = $rec_count[0]->tot;

        /**
         * check if shipping classes need to be created
         */
        $this->create_shipping_classes();

        /**
         * Check if there are products to import
         */

        if ($count > 0) {
            $page_rec           = $wpdb->get_results("select * from " . $wpdb->prefix . "dropship_import_cron_dtl_temp");
            $pagenumber         = $page_rec[0]->page_no;
            $sql_delete_data    = $wpdb->prepare("DELETE FROM `" . $wpdb->prefix . "dropship_import_cron_dtl_temp` WHERE page_no=%d", $pagenumber);
            $wpdb->query($sql_delete_data);
            $feed_pages         = $record[0]->total_feed;

            /**
             * Start import / update process
             */
            $this->ImportProductListCronNow($language, $license_no, $domain, $pagenumber);

            /**
             * Update database when import/update is done
             */

            if ($pagenumber >= $feed_pages) {
                $sql_delete_data = $wpdb->prepare("DELETE FROM `" . $wpdb->prefix . "dropship_import_cron_dtl` WHERE page_no=%d", $pagenumber);
                $wpdb->query($sql_delete_data);
            }
            else {
                /**
                 * Add new page to import
                 */
                $next_page = $pagenumber + 1;

                $sql_support = $wpdb->prepare("UPDATE `" . $wpdb->prefix . "dropship_import_cron_dtl` SET `page_no`='" . $next_page . "', `modified_on`='" . date("Y-m-d H:i:s") . "'");
                $wpdb->query($sql_support);

                $sql = "SELECT COUNT(*) as totrec FROM " . $wpdb->prefix . "dropship_import_cron_dtl";
                $tot_rec = (int)$wpdb->get_var($sql);
                if ($tot_rec > 0) {
                    $created_on = date("Y-m-d H:i:s");
                    $wpdb->insert($wpdb->prefix . "dropship_import_cron_dtl_temp",
                        array(
                            'page_no' => $next_page,
                            'created_on' => $created_on
                        )
                    );
                }
            }

        }

    }

    public function ImportProductListCronNow($language, $license_no, $domain, $i)
    {
        global $wpdb;
        include_once(ABSPATH . 'wp-admin/includes/upgrade.php');

        include_once plugin_dir_path(dirname(__FILE__)) . 'core/mantella_import_update_method.php';
        $dium = new DropshipImportUpdateMethod($this->data);

        $record = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "license_key");
        $this->mantella_ean_number_show_in_front = $record[0]->ean_show_in_front;
        $this->mantella_brand_show_in_front      = $record[0]->brand_show_in_front;
        $this->brand_with_title_status           = $record[0]->brand_with_title_status;
        $this->product_attributes_status         = $record[0]->product_attributes_status;
        $lang = strtolower($language);
        $this->import_settings = unserialize($record[0]->import_settings);

        /**
         * Get products
         */
        $inner_url = $this->api_domain . "feeds/" . $domain . "/" . $license_no . "/feed/" . $i . "?perpage=" . $record[0]->perpage;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_URL, $inner_url);
        $inner_result = curl_exec($ch);
        curl_close($ch);
        $jsondata = json_decode($inner_result, true);

        /**
         * Start import/ update
         */

        if (!empty($jsondata['data'])) {
            $k = 0;
            $b = 0;
            $finalArr = [];
            if ($k == 0 && $i == 1) {
                $log_status = 'Import started';
                $this->logImportedData($log_status, $record, $i, '', '', '', 'manual_import');
            }

            $imported_arr = [];
            $updated_arr  = [];

            foreach ($jsondata['data'] as $data) {
                $check_title_id = $wpdb->get_var("SELECT post_id FROM $wpdb->postmeta WHERE meta_key = '_mantella_ean_number' AND meta_value = '" . $data['ean'] . "'");

                /** fix for plugin updates from older version */
                if (empty($check_title_id))
                    $check_title_id = $wpdb->get_var("SELECT post_id FROM $wpdb->postmeta WHERE meta_key = 'post_ean' AND meta_value = '" . $data['ean'] . "'");
                if (empty($check_title_id))
                    $check_title_id = $wpdb->get_var("SELECT post_id FROM $wpdb->postmeta WHERE meta_key = 'mantella_ean_code' AND meta_value = '" . $data['ean'] . "'");
                /** end fix */

                if (empty($check_title_id)) {
                    try {
                        /**
                         * Create product
                         */

                        //condition to append brand with product name
                        if ($this->brand_with_title_status == 'Yes') {
                            $this->product_name = ucfirst($data['brand'] . ' ' . strtolower($data['name_' . $lang]));
                        } else {
                            $this->product_name = ucfirst(strtolower($data['name_' . $lang]));
                        }

                        $this->product_desc = '';
                        $this->post_excerpt = '';

                        if ($this->import_settings['description_status'] == 'Yes') {
                            $this->product_desc = $data['description'][$lang];
                        }

                        $this->post = array(
                            'post_content'  => $this->product_desc,
                            'post_status'   => "draft",             // set to draft untill all data is imported
                            'post_title'    => $this->product_name,
                            'post_type'     => "product",
                        );

                        if ($this->import_settings['shortdescription_status'] == 'Yes') {

                            //Meeples has short_description. TOM BV does not
                            if( isset( $data['short_description'][$lang] ) )
                                $this->post['post_excerpt'] = $data['short_description'][$lang];
                            else {
                                if ($lang == 'nl')
                                    $desc_arr = explode("Specificaties", $data['description'][$lang], 2);
                                elseif ($lang == 'en')
                                    $desc_arr = explode("Specifications", $data['description'][$lang], 2);
                                elseif ($lang == 'de')
                                    $desc_arr = explode("Spezifikationen", $data['description'][$lang], 2);

                                $prod_excerpt = $desc_arr[0];
                                $prod_excerpt = str_replace( '<strong>', '', $prod_excerpt);
                                $prod_excerpt = str_replace( '</strong>', '', $prod_excerpt);
                                $this->post['post_excerpt'] = $prod_excerpt;
                            }
                        }

                        $this->post_id = wp_insert_post($this->post, true);
                        //post id stored for the tracking
                        array_push($imported_arr, $this->post_id);

                        /**
                         * Category
                         */
                        if ($this->import_settings['categories_status'] == 'Yes') {
                            wp_set_object_terms($this->post_id, $data['categories']['parent_name_' . $lang], 'product_cat');
                            $childcat = isset($data['categories']['child_name_' . $lang]) ? $data['categories']['child_name_' . $lang] : "";
                            wp_insert_term($childcat, 'product_cat', array('parent' => term_exists($data['categories']['parent_name_' . $lang], 'product_cat')['term_id']));
                            wp_set_object_terms($this->post_id, [$data['categories']['parent_name_' . $lang], $childcat], 'product_cat', true);
                            wp_set_object_terms($this->post_id, 'simple', 'product_type');
                        }

                        /**
                         * filter data entry in wc_product_meta_lookup
                         */

                        if ($this->post_id != '' && is_numeric($this->post_id)) {

                            $lookuptable = $wpdb->prefix . 'wc_product_meta_lookup';
                            $recordlookup = $wpdb->get_results("SELECT `product_id` FROM " . $lookuptable . " where product_id = " . $this->post_id);

                            if (count($recordlookup) == 0) {

                                if( $data['seller'] == 'meeples' )
                                    $sku = 'M-'.$data['meeples_id'];
                                elseif( $data['seller'] == 'twm' )
                                    $sku = 'TWM-'.$data['twm_id'];
                                else
                                    $sku = '';

                                $lookupdata = array(
                                    'product_id' => $this->post_id,
                                    'sku'        => $sku,
                                    'min_price'  => $data['consumer_price'],
                                    'max_price'  => $data['consumer_price']);

                                $wpdb->insert($lookuptable, $lookupdata);
                            }

                            /**
                             * Pricing with check for sale_price
                             */
                            update_post_meta($this->post_id, '_sale_price', '');
                            update_post_meta($this->post_id, '_price', $data['consumer_price']);
                            update_post_meta($this->post_id, '_regular_price', $data['consumer_price']);

                            /**
                             * Update other information
                             */
                            update_post_meta($this->post_id, '_visibility', 'visible');
                            update_post_meta($this->post_id, 'post_ean', $data['ean']);
                            update_post_meta($this->post_id, 'total_sales', '0');
                            update_post_meta($this->post_id, '_downloadable', "");
                            update_post_meta($this->post_id, '_purchase_note', "");
                            update_post_meta($this->post_id, '_featured', "no");
                            update_post_meta($this->post_id, '_weight', "");
                            update_post_meta($this->post_id, '_length', "");
                            update_post_meta($this->post_id, '_width', "");
                            update_post_meta($this->post_id, '_height', "");
                            update_post_meta($this->post_id, '_sku', $data['twm_id']);
                            update_post_meta($this->post_id, '_sold_individually', "");
                            update_post_meta($this->post_id, '_manage_stock', "Yes");
                            update_post_meta($this->post_id, '_backorders', "no");
                            update_post_meta($this->post_id, '_mantella_ean_number_show_in_front', $this->mantella_ean_number_show_in_front);
                            update_post_meta($this->post_id, '_mantella_ean_number', $data['ean']);
                            update_post_meta($this->post_id, '_mantella_brand_show_in_front', $this->mantella_brand_show_in_front);
                            update_post_meta($this->post_id, '_dropshiptool_seller', $data['seller']);
                            update_post_meta($this->post_id, '_dropshiptool_ean', $data['ean']);
                            if ($this->import_settings['brand_status'] == 'Yes') {
                                update_post_meta($this->post_id, '_mantella_brand', $data['brand']);
                                update_post_meta($this->post_id, '_dropshiptool_brand', $data['brand']);
                            }
                            update_post_meta($this->post_id, '_tax_status', "taxable");
                            update_post_meta($this->post_id, '_tax_class', "");
                            update_post_meta($this->post_id, '_virtual', "no");
                            update_post_meta($this->post_id, '_tax_class', "");
                            update_post_meta($this->post_id, '_wc_review_count', 0);
                            update_post_meta($this->post_id, '_wc_average_rating', 0);

                            /**
                             * Update stock information
                             */
                            update_post_meta($this->post_id, '_stock', $data['stock']);

                            if ($data['stock'] < 1) {
                                update_post_meta($this->post_id, '_stock_status', 'outofstock');

                                $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                                $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 0 ), array( 'product_id' =>  $this->post_id) );
                            }
                            else{
                                update_post_meta($this->post_id, '_stock_status', 'instock');

                                $table_attr_lookup   = $wpdb->prefix . 'wc_product_attributes_lookup';
                                $update_stock_lookup = $wpdb->update( $table_attr_lookup, array( 'in_stock' => 1 ), array( 'product_id' =>  $this->post_id) );
                            }

                            /**
                             * Set shipping classes
                             */
                            if ($this->import_settings['shipping_class_status'] == 'Yes') {
                                if( $data['seller'] == 'meeples' )
                                {
                                    wp_set_post_terms($this->post_id, array("meeples_standard"), 'product_shipping_class');
                                } else {
                                    if ($data['shipping_size'] == "XS")
                                        wp_set_post_terms($this->post_id, array("tom_bv_xs"), 'product_shipping_class');
                                    elseif ($data['shipping_size'] == "S")
                                        wp_set_post_terms($this->post_id, array("tom_bv_s"), 'product_shipping_class');
                                    elseif ($data['shipping_size'] == "M")
                                        wp_set_post_terms($this->post_id, array("tom_bv_m"), 'product_shipping_class');
                                    elseif ($data['shipping_size'] == "L")
                                        wp_set_post_terms($this->post_id, array("tom_bv_l"), 'product_shipping_class');
                                    elseif ($data['shipping_size'] == "XL")
                                        wp_set_post_terms($this->post_id, array("tom_bv_xl"), 'product_shipping_class');
                                }
                            }

                            /**
                             * Attribute Product Details
                             */
                            if ($this->import_settings['product_attributes_status'] == 'Yes')
                            {
                                /** preparations for attribute creation
                                Check if the custom attributes has been set
                                 */

                                if( $this->import_settings['attr_choose'] == 'No')
                                    $array_attr = NULL;
                                else {
                                    $expl       = explode('|', $this->import_settings['custom_attributes']);
                                    $array_attr = array();
                                    foreach( $expl as $ex ) { $array_attr[] = $ex; }
                                }

                                /** check if brand also needs to be added */
                                if ($this->import_settings['store_brand_as_attribute'] == 'Yes' && isset($data['brand']) && $data['brand'] != '') {
                                    $brand_attr['id']       = $this->post_id;
                                    $brand_attr['name_en']  = 'Brand';
                                    $brand_attr['name_nl']  = 'Merk';
                                    $brand_attr['name_de']  = 'Marke';
                                    $brand_attr['value_en'] = $data['brand'];
                                    $brand_attr['value_nl'] = $data['brand'];
                                    $brand_attr['value_de'] = $data['brand'];

                                    if ($data['attributes']) {
                                        array_push($data['attributes'], $brand_attr);
                                    } else {
                                        $data['attributes'] = [];
                                        array_push($data['attributes'], $brand_attr);
                                    }
                                }

                                /** if global is set */
                                if ($this->import_settings['attribute_type'] == 'global') {

                                    $response = $this->importProductAttributes($data, $lang, $this->post_id, $b, $array_attr);
                                    $b = $response['b'];

                                    if (is_array($response['serializeArr']))
                                        $finalArr = array_merge($finalArr, $response['serializeArr']);

                                } /** if local is set */
                                else {
                                    if (isset($data['attributes'])) {
                                        $product_attributes = NULL;
                                        $ai = 0;

                                        /** process attributes */
                                        foreach ($data['attributes'] as $attr) {

                                            if( $array_attr != NULL ) {
                                                if( !in_array(strtolower($attr['name_' . $lang]), $array_attr) ) {
                                                    continue;
                                                }
                                            }

                                            $product_attributes[$ai] = array(
                                                'name'          => $attr['name_' . $lang],
                                                'value'         => $attr['value_' . $lang],
                                                'position'      => 1,
                                                'is_visible'    => 1,
                                                'is_variation'  => 1,
                                                'is_taxonomy'   => 0
                                            );
                                            $ai++;
                                        }

                                        update_post_meta($this->post_id, '_product_attributes', $product_attributes);
                                    }
                                }
                                /**
                                 */
                            }

                            /**
                             * Import Images
                             */
                            if (isset($data['media'])) // upload multiple images
                            {
                                $gallery_img = NULL;
                                $c = 1;
                                foreach ($data['media'] as $media) {
                                    if ($media['type'] == 'image') {
                                        $attach_id_img = $this->uploadProductImagesCronNow($this->post_id, $media['url'], $data['seller']);
                                        if ($c > 1) {
                                            if ($c == 2)
                                                $gallery_img = $attach_id_img;
                                            else
                                                $gallery_img = $gallery_img . ',' . $attach_id_img;
                                        } else
                                            set_post_thumbnail($this->post_id, $attach_id_img);
                                    }
                                    $c++;
                                }
                                update_post_meta($this->post_id, '_product_image_gallery', $gallery_img);
                            }

                            //insert addtional data starts
                            $attach_id = get_post_meta($this->post_id, "_thumbnail_id", true);
                            add_post_meta($this->post_id, '_thumbnail_id', $attach_id);

                            unset($gallery_img);
                            unset($attach_id_img1);
                            if ($attach_id_img2)
                                unset($attach_id_img2);

                            /**
                             * Publish product
                             */
                            $post_update['ID'] = $this->post_id;
                            if ($this->import_settings['added_product_status'] == 'draft') {
                                $post_update['post_status'] = 'draft';
                            } else {
                                $post_update['post_status'] = 'publish';
                            }

                            wp_update_post($post_update);

                        }

                    } catch (Exception $e) {
                        //Log data
                        $log_status = 'Import error';
                        $this->logImportedData($log_status, $record, $i, $e->getMessage(), $imported_arr, $updated_arr, 'manual_import');
                    }

                    /**
                     * Reset product cache/transients
                     */

                    wc_delete_product_transients($this->post_id);

                }
                else {
                    if (get_post_status($check_title_id) != 'trash') {
                        array_push($updated_arr, $check_title_id);

                        $response = $dium->update_import_method_from_cron_new($lang, $record, $data, $check_title_id, $b);

                        if ($this->import_settings['attribute_type'] == 'global' && $this->import_settings['product_attributes_status'] == 'Yes')
                        {
                            $b = $response['b'];

                            if (is_array($response['serializeArr']) && (count($response['serializeArr']) != 0) )
                                $finalArr = array_merge($finalArr, $response['serializeArr']);
                        }
                    }
                }
                unset($jsondata['data'][$k]);
                $k++;
            }
            if ($record[0]->total_feed != $i) {
                $log_status = '';
                $sql_log = $this->logImportedData($log_status, $record, $i, '', $imported_arr, $updated_arr, 'manual_import');
            }

            //set all option in options table
            if ($this->import_settings['attribute_type'] == 'global') {
                $this->setAttributesOptions($finalArr);
            }
            //import finished
            if ($record[0]->total_feed == $i) {
                $log_status = 'Import finished';
                $this->logImportedData($log_status, $record, $i, '', $imported_arr, $updated_arr, 'manual_import');
            }
        }
    }

    //This function is used to import the attributes
    public function importProductAttributes($data, $lang, $post_id, $b, $array_attr = NULL)
    {
        global $wpdb;
        $serializeArr = [];

        if (isset($data['attributes'])) {

            $product_attributes = NULL;
            $prAttributestable = $wpdb->prefix . 'woocommerce_attribute_taxonomies';
            $a = 0;

            foreach ($data['attributes'] as $attr) {

                if( $array_attr != NULL ) {
                    if( !in_array(strtolower($attr['name_' . $lang]), $array_attr) ) {
                        continue;
                    }
                }

                $product_attributes[$a] = array(
                    'name'          => 'pa_' . sanitize_title($attr['name_' . $lang]),
                    'value'         => '', //$attr['value_'.$lang]
                    'position'      => 1,
                    'is_visible'    => 1,
                    'is_variation'  => 1,
                    'is_taxonomy'   => 1
                );

                //attribute entry
                $recordattr = $wpdb->get_results("SELECT `attribute_name` FROM " . $prAttributestable . " where attribute_name = '" . sanitize_title($attr['name_' . $lang]) . "'");

                if (count($recordattr) == 0) {

                    $prAttributesdata = array(
                        'attribute_name'    => sanitize_title($attr['name_' . $lang]),
                        'attribute_label'   => $attr['name_' . $lang],
                        'attribute_type'    => 'select',
                        'attribute_orderby' => 'id',
                        'attribute_public'  => 0);

                    $wpdb->insert($prAttributestable, $prAttributesdata);
                    $attributeid = $wpdb->insert_id;

                    $arrwithattrid = array_merge(array('attribute_id' => $attributeid), $prAttributesdata);
                    $serializeArr[$b] = (object)$arrwithattrid;

                    $b++;

                }
                $a++;
                $this->insertTermsTaxonomy($attr, $lang, $post_id);

            }
            update_post_meta($post_id, '_product_attributes', $product_attributes);
        }
        return ['b' => $b, 'serializeArr' => $serializeArr];
    }

    //Set option values
    public function setAttributesOptions($finalArr)
    {
        if (count($finalArr) > 0) {

            $attroptionsval     = get_option('_transient_wc_attribute_taxonomies');
            $appendwithfinalArr = array_filter(array_merge($finalArr, $attroptionsval));
            $serializeArrVal    = sanitize_option('_transient_wc_attribute_taxonomies', $appendwithfinalArr);

            update_option('_transient_wc_attribute_taxonomies', $serializeArrVal);
        }

    }

    //Insert Terms and taxonomy data
    public function insertTermsTaxonomy($attr, $lang, $product_id, $instock = 1)
    {
        global $wpdb;

        $taxonomy = 'pa_' . sanitize_title($attr['name_' . $lang]); // The taxonomy
        $term_name = $attr['value_' . $lang]; // The term "NAME"

        $termstable     = $wpdb->prefix . 'terms';
        $lookuptable    = $wpdb->prefix . 'wc_product_attributes_lookup';
        $recordterm     = $wpdb->get_results("SELECT `term_id` FROM " . $termstable . " where slug = '" . sanitize_title($attr['value_' . $lang]) . "'");

        if (count($recordterm) == 0) {

            //insert data into terms table
            $termsdata = array(
                'name' => $attr['value_' . $lang],
                'slug' => sanitize_title($attr['value_' . $lang]));

            $wpdb->insert($termstable, $termsdata);
            $term_id = $wpdb->insert_id;

            //insert data into taxonomy table
            $termstaxonomytable = $wpdb->prefix . 'term_taxonomy';
            $taxonomydata = array(
                'term_id'   => $term_id,
                'taxonomy'  => 'pa_' . sanitize_title($attr['name_' . $lang]),
                'count'     => 1 );

            $wpdb->insert($termstaxonomytable, $taxonomydata);
            $term_taxonomy_id = $wpdb->insert_id;
            //insert data into taxonomy table
            $this->insertTaxonomyRelation($term_taxonomy_id, $product_id);


            // setup the tabel for filter
            $check_lookup = $wpdb->get_results("SELECT `product_or_parent_id` FROM " . $lookuptable . " 
                                                WHERE product_id = '".$product_id."' 
                                                AND  taxonomy =  'pa_" . sanitize_title($attr['name_' . $lang]) . "'
                                                AND term_id = '".$term_id."'");

            if (count($check_lookup) == 0) {
                $data_array = array(
                    'product_id'            => $product_id,
                    'product_or_parent_id'  => $product_id,
                    'taxonomy'              => 'pa_' . sanitize_title($attr['name_' . $lang]),
                    'term_id'               => $term_id, //add this
                    'is_variation_attribute' => 0,
                    'in_stock'              => $instock);
                $wpdb->insert($lookuptable, $data_array);
            }
            // end setup

            //update count
            $this->updateTermTaxonomyCount($term_id);

        } else {
            $termsrec = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "terms where name = '" . esc_sql($term_name) . "'");

            if ($termsrec) {
                foreach ($termsrec as $termsrecdata) {
                    $term_id = $termsrecdata->term_id;
                    $taxonomyrec = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "term_taxonomy where term_id = '" . $term_id . "'");

                    if ($taxonomyrec) {
                        foreach ($taxonomyrec as $taxonomydata) {
                            if ($taxonomydata->term_taxonomy_id) {
                                $relrec = $wpdb->get_results("SELECT * FROM " . $wpdb->prefix . "term_relationships where term_taxonomy_id = " . $taxonomydata->term_taxonomy_id . " AND object_id = " . $product_id . "");
                                if (count($relrec) == 0) {
                                    $this->insertTaxonomyRelation($taxonomydata->term_taxonomy_id, $product_id);
                                }
                            }
                        }
                    }

                    // setup the tabel for filter
                    $check_lookup = $wpdb->get_results("SELECT `product_or_parent_id` FROM " . $lookuptable . " 
                                                WHERE product_id = '".$product_id."' 
                                                AND  taxonomy =  'pa_" . sanitize_title($attr['name_' . $lang]) . "'
                                                AND term_id = '".$term_id."'");

                    if (count($check_lookup) == 0) {
                        $data_array = array(
                            'product_id' => $product_id,
                            'product_or_parent_id' => $product_id,
                            'taxonomy' => 'pa_' . sanitize_title($attr['name_' . $lang]),
                            'term_id' => $term_id, //add this
                            'is_variation_attribute' => 0,
                            'in_stock' => $instock);
                        $wpdb->insert($lookuptable, $data_array);
                    }
                    // end setup

                    //update count
                    $this->updateTermTaxonomyCount($term_id);
                }
            }
        }
    }

    //set the taxonomy relation
    public function insertTaxonomyRelation($term_taxonomy_id, $product_id)
    {
        global $wpdb;

        $relationaltable = $wpdb->prefix . 'term_relationships';
        $recordlookup = $wpdb->get_results("SELECT `object_id` FROM " . $relationaltable . " where object_id = " . $product_id . " AND term_taxonomy_id = " . $term_taxonomy_id);

        if (count($recordlookup) == 0) {
            $taxonomyreldata = array(
                'object_id' => $product_id,
                'term_taxonomy_id' => $term_taxonomy_id);

            $wpdb->insert($relationaltable, $taxonomyreldata);
        }
    }

    //update count in term_taxonomy table
    public function updateTermTaxonomyCount($term_id)
    {
        global $wpdb;

        $lookuptable        = $wpdb->prefix . 'wc_product_attributes_lookup';
        $termstaxonomytable = $wpdb->prefix . 'term_taxonomy';

        //get the count
        $count_term_array   = $wpdb->get_results("SELECT * FROM " . $lookuptable . " where term_id =  ". $term_id ."");
        $count_term         = count($count_term_array);

        $wpdb->update( $termstaxonomytable, array( 'count' =>  $count_term), array( 'term_id' => $term_id ) );

    }

    //Image Upload function
    public function uploadProductImagesCronNow($post_id, $image_url, $seller = NULL)
    {
        if ($attach_id)
            unset($attach_id);

        $image_name         = basename($image_url);
        $upload_dir         = wp_upload_dir();

        if( $seller == 'meeples'){
            $ch = curl_init($image_url);
            curl_setopt($ch, CURLOPT_HEADER, false);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_BINARYTRANSFER, 1);
            curl_setopt($ch, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US)");
            $image_data = curl_exec($ch);
            curl_close($ch);
        } else {
            $image_data         = file_get_contents($image_url);
        }

        $unique_file_name   = @wp_unique_filename($upload_dir['path'], $image_name);
        $filename           = @basename($unique_file_name);

        if (wp_mkdir_p($upload_dir['path']))
            $file = $upload_dir['path'] . '/' . $filename;
        else
            $file = $upload_dir['basedir'] . '/' . $filename;

        file_put_contents($file, $image_data);

        $wp_filetype = wp_check_filetype($filename, null);
        $attachment = array(
            'post_mime_type'    => $wp_filetype['type'],
            'post_title'        => sanitize_file_name($filename),
            'post_content'      => '',
            'post_status'       => 'inherit'
        );

        $attach_id = wp_insert_attachment($attachment, $file, $post_id);

        require_once(ABSPATH . 'wp-admin/includes/image.php');


        //wp_create_image_subsizes($file, $attach_id);
        wp_generate_attachment_metadata($attach_id, $file);

        unset($image_name);
        unset($image_data);
        unset($unique_file_name);
        unset($filename);
        unset($attachment);
        unset($attach_data);
        unset($file);
        unset($wp_filetype);

        return $attach_id;
    }

    public function create_shipping_classes()
    {
        $term_exists_xs = term_exists('tom_bv_xs', 'product_shipping_class');
        if (empty($term_exists_xs))
            wp_insert_term('TOM BV XS', 'product_shipping_class', array('description' => 'TOM BV Shipping class xs', 'slug' => 'tom_bv_xs',));

        $term_exists_s = term_exists('tom_bv_s', 'product_shipping_class');
        if (empty($term_exists_s))
            wp_insert_term('TOM BV S', 'product_shipping_class', array('description' => 'TOM BV Shipping class s', 'slug' => 'tom_bv_s',));

        $term_exists_m = term_exists('tom_bv_m', 'product_shipping_class');
        if (empty($term_exists_m))
            wp_insert_term('TOM BV M', 'product_shipping_class', array('description' => 'TOM BV Shipping class m', 'slug' => 'tom_bv_m',));

        $term_exists_l = term_exists('tom_bv_l', 'product_shipping_class');
        if (empty($term_exists_l))
            wp_insert_term('TOM BV L', 'product_shipping_class', array('description' => 'TOM BV Shipping class l', 'slug' => 'tom_bv_l',));

        $term_exists_xl = term_exists('tom_bv_xl', 'product_shipping_class');
        if (empty($term_exists_xl))
            wp_insert_term('TOM BV XL', 'product_shipping_class', array('description' => 'TOM BV Shipping class xl', 'slug' => 'tom_bv_xl',));

        $term_exists_meeples = term_exists('meeples_standard', 'product_shipping_class');
        if (empty($term_exists_xl))
            wp_insert_term('Meeples Standaard', 'product_shipping_class', array('description' => 'Meeples standaard verzendkosten', 'slug' => 'meeples_standard',));

        return true;
    }

    //check file exists
    public function checkFileExists($post_id, $image_url)
    {
        $image_name = basename($image_url);
        $image_exists = 0;

        if ($post_id != '') {

            $db_image_url = get_the_post_thumbnail_url($post_id, '');
            if ($db_image_url) {

                $filedata = pathinfo($db_image_url);
                $path_subdir = explode("uploads", $filedata['dirname']);
                $upload_dir = wp_upload_dir();
                $file_path = $upload_dir['basedir'] . $path_subdir[1] . '/' . $image_name;
                chmod($file_path, 0777);

                if (file_exists($file_path)) {
                    $image_exists = 1;
                }
            }
        }

        return $image_exists;

    }

    public function callCronSchedulesAfterPluginActivate()
    {

        add_filter('cron_schedules', array($this, 'mantella_custom_productimport_cronjob_schedule_now'));
        if (!wp_next_scheduled('mantella_cron_job_productimport_now')) {
            wp_schedule_event(time(), 'every_three_min_cron_now', 'mantella_cron_job_productimport_now');
        }
        add_action('mantella_cron_job_productimport_now', array($this, 'mantella_cron_productimport_function_now'));

    }

    public function logImportedData($log_status, $record, $pagenumber, $error = '', $imported_arr, $updated_arr, $log_type = NULL)
    {
        global $wpdb;

        if ($record != '') {
            if ($pagenumber == 1 && $record[0]->total_feed != 0) {
                $total_batches = $record[0]->total_feed;
            } else {
                $total_batches = 0;
            }
            $total_imported_products = $record[0]->perpage * $pagenumber;
        }

        if ($error) {
            $this->custom_log($error, 'a', 'debug');
        }

        if ($imported_arr != '') {
            $per_batch_imported = json_encode($imported_arr);
            $message = count($imported_arr) . ' imported';
        } else {
            $per_batch_imported = '';
            $message = '';
        }
        if ($updated_arr != '') {
            $per_batch_updated = json_encode($updated_arr);
            $message = count($updated_arr) . ' updated';
        } else {
            $per_batch_updated = '';
            $message = '';
        }

        if ($imported_arr != '' && $updated_arr != '') {
            $message = count($imported_arr) . ' imported and ' . count($updated_arr) . ' updated';
        }

        if ($message != '') {
            $this->custom_log($message, 'a', 'debug');
        }

        $created_on = date("Y-m-d H:i:s");
        $loggedinuserid = get_current_user_id();

        $logtable = $wpdb->prefix . 'dropship_track_logs';
        $logarr = array(
            'wp_user_id'              => $loggedinuserid,
            'log_text'                => $error,
            'batch_number'            => $pagenumber,
            'total_batches'           => $total_batches,
            'total_imported_products' => $total_imported_products,
            'per_batch_imported'      => $per_batch_imported,
            'per_batch_updated'       => $per_batch_updated,
            'log_status'              => $log_status,
            'log_type'                => $log_type,
            'log_date'                => $created_on,
            'modified_on'             => $created_on);

        $wpdb->insert($logtable, $logarr);
    }

    public function custom_log($entry, $mode = 'a', $file = 'debug')
    {
        /** Deactivated by dropshiptool **
         *
         * // Get WordPress uploads directory.
         * define( 'WP_DEBUG_LOG', true );
         *
         * $upload_dir = WP_CONTENT_DIR . '/uploads/import_log';
         * chmod($upload_dir, 0777);
         * // If the entry is array, json_encode.
         *
         * if ( is_array( $entry ) ) {
         * $entry = json_encode( $entry );
         * }
         *
         * // Write the log file.
         * $file  = $upload_dir . '/' . $file . '.log';
         * $file  = fopen( $file, $mode );
         * $bytes = fwrite( $file, "\n".current_time( 'mysql' ) . "::" . $entry . "\n" );
         * fclose( $file );
         *
         * return $bytes;
         * */
    }

}