<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class DropshipTools
{
    private $plugin_name;
    private $version;
    private $license_no;
    private $domain;
    private $api_domain;
    private $data;

    public function __construct($dataArray)
    {
        $this->plugin_name  = $dataArray['plugin_name'];
        $this->version      = $dataArray['plugin_version'];
        $this->domain       = $dataArray['plugin_domain'];
        $this->api_domain   = $dataArray['api_domain'];
        $this->data         = $dataArray;
    }

    public function deleteAllEans()
    {
        $all = true;
        $this->deleteEans($all);

        return 'success';
    }

    public function deleteEans($all = false)
    {
        global $wpdb, $woocommerce, $wp_version;
        include_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

        $delete_success = 0;
        $delete_error = 0;
        $skipped = 0;

        $this->logDeletedProduct( NULL, 'Delete product batch started.');

        $record = $wpdb->get_results("select * from " . $wpdb->prefix . "license_key");

        $license_no = $record[0]->license_no;
        $id_array   = $wpdb->get_results("SELECT * FROM $wpdb->postmeta WHERE meta_key = '_mantella_ean_number' OR meta_key = 'post_ean' OR meta_key = 'mantella_ean_code' ", 'ARRAY_A');

        foreach( $id_array as $data )
        {
            //delete all products
            if( $all )
            {
                $delete = $this->deleteFunction($data['post_id']);

                if ($delete) {
                    $this->logDeletedProduct($data['meta_value']);
                    $delete_success++;
                } else
                    $delete_error++;

                //delete product not in personal feed
            } else
            {
                $json_url = $this->api_domain . "feeds/" . $this->domain . "/" . $license_no . "/info/" . $data['meta_value'];

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($ch, CURLOPT_URL, $json_url);
                $get_pages_result = curl_exec($ch);
                curl_close($ch);
                $info = json_decode($get_pages_result, true);

                if( isset($info['data']['access']) && $info['data']['access'] == 'denied')
                {
                    if( isset($info['data']['reason']))
                    {
                        if( $info['data']['reason'] == 'EAN not found' OR $info['data']['reason'] == 'EAN not found in personal feed') {
                            $delete = $this->deleteFunction($data['post_id']);

                            if ($delete) {
                                $this->logDeletedProduct($data['meta_value']);
                                $delete_success++;
                            } else
                                $delete_error++;
                        }
                    }
                } else
                {
                    $skipped++;
                }
            }
        }

        $return['status']       = true;
        $return['d_success']    = $delete_success;
        $return['d_error']      = $delete_error;
        $return['skipped']      = $skipped;

        $this->logDeletedProduct(NULL, $return);

        return 'success';
    }

    public function deleteAttr()
    {
        global $wpdb, $woocommerce, $wp_version;
        include_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

        $this->logDeletedProduct( NULL, 'Delete attributes function started.', 'attributes', 'delete_batch_started');

        //Delete global attributes
        $woocommerce_attribute_taxonomies   = $wpdb->prefix . 'woocommerce_attribute_taxonomies';
        $ids                                = $wpdb->get_results("SELECT * FROM $woocommerce_attribute_taxonomies", 'ARRAY_A');

        foreach( $ids as $id ) {
            wc_delete_attribute( $id['attribute_id'] );
        }

        //Delete attribute connections
        $postmeta   = $wpdb->prefix . 'postmeta';
        $wpdb->query("DELETE FROM $postmeta WHERE meta_key = '_product_attributes';");

        //Delete WP Terms
        $wp_terms                = $wpdb->prefix . 'terms';
        $wp_term_taxonomy        = $wpdb->prefix . 'term_taxonomy';
        $wp_term_relationships   = $wpdb->prefix . 'term_relationships';

        $query1 = $wpdb->query("DELETE FROM $wp_terms WHERE term_id IN (SELECT term_id FROM $wp_term_taxonomy WHERE taxonomy LIKE 'pa_%');");
        $wpdb->query("DELETE FROM $wp_term_taxonomy WHERE taxonomy LIKE 'pa_%';");
        $wpdb->query("DELETE FROM $wp_term_relationships WHERE term_taxonomy_id not IN (SELECT term_taxonomy_id FROM $wp_term_taxonomy);");

        $this->logDeletedProduct( NULL, 'Delete attributes function finished.', 'attributes', 'delete_batch_is_done');

        return 'success';
    }

    private function deleteFunction( $id )
    {

        $product = wc_get_product($id);

        //check
        if(empty($product))
            return false; //could not be deleted

        //delete images
        $featured_image_id  = $product->get_image_id();
        $image_galleries_id = $product->get_gallery_image_ids();

        if( !empty( $featured_image_id ) ) {
            wp_delete_post( $featured_image_id, true );
        }

        if( !empty( $image_galleries_id ) ) {
            foreach( $image_galleries_id as $single_image_id ) {
                wp_delete_post( $single_image_id, true );
            }
        }


        if ($product->is_type('variable'))
        {
            foreach ($product->get_children() as $child_id)
            {
                $child = wc_get_product($child_id);
                $child->delete(true);
            }
        }
        elseif ($product->is_type('grouped'))
        {
            foreach ($product->get_children() as $child_id)
            {
                $child = wc_get_product($child_id);
                $child->set_parent_id(0);
                $child->save();
            }
        }

        $product->delete(true);
        $result = $product->get_id() > 0 ? false : true;


        if (!$result)
            return false; //could not be deleted

        // Delete parent product transients.
        if ($parent_id = wp_get_post_parent_id($id))
            wc_delete_product_transients($parent_id);

        return true;
    }

    private function logDeletedProduct( $ean = NULL, $info = NULL, $sort = NULL, $status = NULL)
    {
        global $wpdb;

        $created_on     = date("Y-m-d H:i:s");
        $loggedinuserid = get_current_user_id();

        if($sort == 'attributes')
        {
            $log_text   = $info;
            $log_status = $status;
        } else {
            if ($ean != NULL) {
                $log_text = 'Cleanup tool: Deleted product with EAN ' . $ean;
                $log_status = 'product_deleted';
            } elseif (isset($info['d_success'])) {
                $log_text = 'Delete product batch finished. Deleted ' . $info['d_success'] . ' products. Skipped ' . $info['skipped'] . ' products. ' . $info['d_error'] . ' errors.';
                $log_status = 'delete_batch_is_done';
            } else {
                $log_text = $info;
                $log_status = 'delete_batch_started';
            }
        }

        $logtable = $wpdb->prefix . 'dropship_track_logs';
        $logarr = array(
            'wp_user_id'              => $loggedinuserid,
            'log_type'                => 'tools_function_delete_product',
            'log_text'                => $log_text,
            'log_status'              => $log_status,
            'log_date'                => $created_on,
            'modified_on'             => $created_on);

        $wpdb->insert($logtable, $logarr);
    }
}
